<?php
/******************************************************************************
 * DolphinPanel - WordPress Option Page Freamwork
 *****************************************************************************/

/*  Copyright hiromasa  (email : h1romas4@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/******************************************************************************
 * WordPress Interface Define
 *****************************************************************************/

if(!(defined('ABSPATH') && defined('WPINC'))) return;

/******************************************************************************
 * Dolphin Class Define
 *****************************************************************************/

/**
 * DolphonPanel のリスナーインターフェースです。
 * 管理画面からの値制御サービスを DolphonPanel でうけるクラスは
 * 本インターフェースを implements し各メソッドを実装後、そのインスタンスを
 * add_dolphin 関数で DolphinController に登録してください。
 *
 * @OriginalName Dolphinable
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
interface Dolphinable {
    
    /**
     * 引数 DolphinForm インスタンスに対して管理画面のフォームと変数のバインド
     * を設定してください。
     *
     * @param DolphinForm $form
     */
    public function onActive(DolphinForm $form);
    
    /**
     * 管理画面のユーザ操作によりフォームに値が設定された時点で呼び出されます。
     * 本メソッドに true を返却した場合、その値が変数にバインドされます。
     * false を返却した場合は値は破棄され元の状態に戻ります。
     *
     * 値検査を実装し DolphinForm インスタンスに対してユーザ対話用のメッセージ
     * を設定します。
     *
     * @param DolphinForm $form
     * @return boolean
     */
    public function onBind(DolphinForm $form);
    
    /**
     * プラグインが管理画面から非活性化された際の動作を実装します。
     * もし、データベース上から DolphinPanel により生成された管理画面の設定
     * 値を削除したい場合は、remove_dolphin 関数を呼び出すことでデータベース
     * からの値削除が行われます。
     */
    public function onDispose();
    
}

/**
 * DolphonPanel を管理するシングルトンクラスです。 複数の DolphonPanel を管理
 * することができます。
 *
 * インスタンスが必要な場合は、getInstance() メソッドを呼び出し唯一のインス
 * タンスを取得してください。
 *
 * @OriginalName DolphinController
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
class DolphinController {
    
    private static $INSTANCE = null;
    
    private $panels = array();
    private $deactivate = array();
    
    /**
     * コンストラクタです。
     * シングルトンクラスのため private に設定されています。
     *
     * @return DolphinController
     */
    private function __construct() { }
    
    /**
     * インスタンスを返却するファクトリーメソッドです。
     *
     * @return DolphinController
     */
    static public function getInstance() {
        
        if(self::$INSTANCE == null) {
            self::$INSTANCE = new DolphinController();
        }
        return self::$INSTANCE;
        
    }
    
    /**
     * DolphinPanel のインスタンスを作成しリスナーを登録します。
     *
     * @param DolphinPanel $panel
     */
    public function add(Dolphinable $listener, $index, $role) {
        
        if(isset($this->panels[$index])) {
            throw new Exception(
                "IllegalArgumentException ($index)");
        }
        
        $this->panels[$index] =
            new DolphinPanel($listener, $index, $role);
        
    }
    
    /**
     * シリアライズされて登録された値をデータベースから削除します。
     *
     * @param DolphinPanel $panel
     */
    public function remove($index) {
        
        return delete_option($index);
        
    }
    
    /**
     * WordPress の init イベント時に、登録された DolphinPanel の初期化を
     * 行います。
     *
     * このメソッドは WordPress とのインターフェースに使われるために公開
     * されています。
     *
     */
    public function notify() {
        
        foreach($this->panels as $panel) {
            $panel->init();
        }
        
    }
    
}

/**
 * 登録されたリスナーと WordPress 管理画面のインターフェースを行うクラスです。
 *
 * 管理画面のページ１つごとに１つのインスタンスが DolphinController により
 * 生成されます。
 *
 * @OriginalName DolphinPanel
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
class DolphinPanel {
    
    private $listener;
    private $listenerName;
    private $form;
    private $index;
    private $file;
    private $role;
    
    /**
     * コンストラクタです。
     *
     * @param Dolphinable $dolphinable
     * @return DolphinPanel
     */
    public function __construct(Dolphinable $listener, $index, $role) {
        
        $ref = new ReflectionClass($listener);
        
        $this->listener = $listener;
        $this->listenerName = $ref->getName();
        $this->index = $index;
        $this->role = $role;
        $this->file = $ref->getFileName();
        $this->form = new DolphinForm($index);
        
        add_action('admin_menu', array($this, 'addAdminHook'));
        
    }
    
    /**
     * init(1) のタイミングで動作する初期化処理です。
     *
     * データベースから値を取得しリスナーから得たフォームの構造を元に値を
     * 初期バインドします。
     * また、データベースに値がない場合もしくはフォームの構造が変更されている
     * 場合はリスナーで設定された値でデータベースを更新します。
     *
     * このメソッドは WordPress とのインターフェースに使われるために公開
     * されています。
     *
     */
    public function init() {
        
        // リスナーにフォームの構造の作成を依頼
        $this->listener->onActive($this->form);
        // フォームの構造からバインドされた変数値を取得
        $values = $this->exportListener();
        
        // 初回起動ならオプションを保存しバインドはしない
        $option = get_option($this->index);
        if(empty($option)) {
            add_option($this->index, $values);
            return;
        }
        // フォームの構造が変更されていたらアップデート
        $update = false;
        // フォームの形が変更された（追加）
        $diff = array_diff_assoc($values, $option);
        if(count($diff) != 0) {
            $update = true;
            foreach($diff as $name => $value) {
                $option[$name] = $value;
            }
        }
        // フォームの形が変更された（削除）
        $diff = array_diff_assoc($option, $values);
        if(count($diff) != 0) {
            $update = true;
            foreach($diff as $name => $value) {
                unset($option[$name]);
            }
        }
        if($update) {
            update_option($this->index, $option);
        }
        
        // オプション値をリスナーにバインドする
        $this->importListener($option);
        
    }
    
    /**
     * 管理画面が表示された場合の処理です。ユーザ操作による値の変更を監視し、
     * オプションの保存と値のバインドを行います。
     *
     * このメソッドは WordPress とのインターフェースに使われるために公開
     * されています。
     */
    public function handshake() {
        
        // データベースから値を取得しフォームに設定する
        $option = get_option($this->index);
        $this->form->setFormValue($option);
        
        // ユーザからの更新通知があれば値を取得してリスナーに通知する
        if($_SERVER["REQUEST_METHOD"] == "POST") {
            // POST 値を確認してフォームに変換
            check_admin_referer("update-options-{$this->index}");
            foreach($this->form->getComponentNames() as $name) {
                $this->form->setComponentUserValue($name, $_POST[$name]);
            }
            // リスナーに値をバインドする
            $values = $this->form->getFormValue();
            $this->importListener($values);
            // リスナーの onBind を呼び出し値の値検査を行う
            if($this->listener->onBind($this->form)) {
                // データベースのアップデートと値のバインドを行う。
                update_option($this->index, $values);
            } else {
                // 値検査失敗の場合は値を戻す
                $this->importListener($option);
            }
        }
        
        // フォームの値を出力する
        $this->form->echoView();
        
    }
    
    /**
     * 管理画面が非活性になった場合の処理です。
     *
     * このメソッドは WordPress とのインターフェースに使われるために公開
     * されています。
     */
    public function dispose() {
        
        $this->listener->onDispose();
        
    }
    
    /**
     * 管理画面のコールバックメソッドを登録します。
     *
     * このメソッドは WordPress とのインターフェースに使われるために公開
     * されています。
     *
     */
    public function addAdminHook() {
        
        add_options_page(
            $this->index
            , $this->index
            , $this->role
            , $this->file
            , array($this, 'handshake')
        );
        
        register_deactivation_hook(
            $this->file
            , array($this, 'dispose'));
        
    }
    
    /**
     * リスナーから値を取得します。
     *
     */
    private function exportListener() {
        
        $vos = array();
        foreach($this->form->getComponentNames() as $name) {
            $propertyName = substr($name, 1, strlen($name));
            try {
                $property =
                  new ReflectionProperty($this->listenerName, $propertyName);
            } catch(ReflectionException $e) {
                throw new Exception("IllegalArgumentException ($name)");
            }
            $value = $property->getValue($this->listener);
            $values = array();
            if(!is_array($value)) {
                $values[0] = $value;
            } else {
                $values = $value;
            }
            $vos[$name] = $values;
        }
        
        return $vos;
        
    }
    
    /**
     * 値をリスナーに設定します。
     *
     * @param $values
     */
    private function importListener($values) {
        
        foreach($this->form->getComponentNames() as $name) {
            $propertyName = substr($name, 1, strlen($name));
            $property =
                new ReflectionProperty($this->listenerName, $propertyName);
            $set = '';
            if(!is_array($property->getValue($this->listener))) {
                $set = $values[$name][0];
            } else {
                $set = $values[$name];
            }
            $property->setValue($this->listener, $set);
        }
        
    }
    
}

/**
 * DolphinPanel に表示されるフォームを表現するクラスです。
 * DolphinComponent を継承したコンポーネントを add メソッドで追加し、入力用の
 * HTML フォームのレイアウトとその管理を行います。
 *
 * @OriginalName DolphinForm
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
class DolphinForm {
    
    private $index;
    private $title;
    private $messages;
    private $components;
    
    /**
     * コンストラクタです。
     *
     * @return DolphinForm
     */
    public function __construct($index) {
        
        $this->index = $index;
        $this->title = 'untitled';
        $this->messages = array();
        $this->components = array();
        
    }
    
    /**
     * DolphinComponent コンポーネントを管理画面に追加します。
     * 呼び出し順に管理画面上からレイアウトされます。
     *
     * @param DolphinComponent $component
     */
    public function add(DolphinComponent $component) {
        
        $name = $component->getName();
        if(isset($this->components[$name])) {
            throw new Exception("IllegalArgumentException ($name)");
        }
        $this->components[$name] = $component;
        
    }
    
    /**
     * 管理画面のタイトルを設定します。
     *
     * @param string $title
     */
    public function setTitle($title) {
        
        $this->title = $title;
        
    }
    
    /**
     *
     * @param $message
     * @return unknown_type
     */
    public function addMessage($message) {
        
        array_push($this->messages, $message);
        
    }
    
    /**
     * 管理画面のフォームを登録されたコンポーネントとともに出力します。
     *
     */
    public function echoView() {
        
        if(count($this->messages) != 0) {
            echo '<div id="message" class="updated fade">' . "\n";
            foreach($this->messages as $message) {
                echo "<p><strong>{$message}</strong></p>" . "\n";
            }
            echo '</div>' . "\n";
        }
        
        echo '<div class="wrap">' . "\n";
        echo  "<h2>{$this->title}</h2>" . "\n";
        echo  '<form method="post">' . "\n";
        wp_nonce_field("update-options-{$this->index}");
        
        echo  '<table class="form-table">' . "\n";
        foreach($this->components as $component) {
            echo '<tr valign="top">' . $component->getView() . '</tr>' . "\n";;
        }
        echo '</table>' . "\n";
        echo '<p class="submit">' . "\n";
        echo '<input type="submit" name="Submit" class="button-primary"';
        echo 'value="' . _('Save Changes') . '" />' . "\n";
        echo '</p>' . "\n";
        echo '</form>' . "\n";
        echo '</div>' . "\n";
        
    }
    
    /**
     * フォームコンポーネントから値を取得します。
     *
     */
    public function getFormValue() {
        
        $values = array();
        foreach($this->getComponentNames() as $name) {
            $values[$name] = $this->getComponentValue($name);
        }
        return $values;
        
    }
    
    /**
     * 値をフォームコンポーネントに登録します。
     *
     */
    public function setFormValue($values) {
        
        foreach($this->getComponentNames() as $name) {
            $this->setComponentValue($name, $values[$name]);
        }
        
    }
    
    /**
     * 登録されたコンポーネントの名称を返却します。
     *
     * @return $names
     */
    public function getComponentNames() {
        
        $names = array();
        foreach($this->components as $component) {
            array_push($names, $component->getName());
        }
        ksort($names);
        return $names;
        
    }
    
    /**
     * 指定したコンポーネントにユーザからの入力値を設定します。
     *
     * @param $name
     * @param $value
     */
    public function setComponentUserValue($name, $value) {
        
        if(!isset($this->components[$name])) {
            throw new Exception("IllegalArgumentException ($name)");
        }
        
        if(!is_array($value)) $value = trim($value);
        $value = stripslashes_deep($value);
        $set = array();
        if(!is_array($value)) {
            $set[0] = $value;
        } else {
            $set = $value;
        }
        $set = $this->components[$name]->doRegular($set);
        if($value !== false) {
            $this->components[$name]->setValue($set);
        }
        
    }
    
    /**
     * 指定したコンポーネントの値を取得します。
     *
     * @param $name
     */
    private function getComponentValue($name) {
        
        if(!isset($this->components[$name])) {
            throw new Exception("IllegalArgumentException ($name)");
        }
        return $this->components[$name]->getValue();
        
    }
    
    /**
     * 指定したコンポーネントに値を設定します。
     *
     * @param $name
     * @param $value
     */
    private function setComponentValue($name, $value) {
        
        if(!isset($this->components[$name])) {
            throw new Exception("IllegalArgumentException ($name)");
        }
        $this->components[$name]->setValue($value);
        
    }
    
}

/**
 * DolphinForm に追加できるコンポーネントの抽象クラスです。
 * 値と指定された変数のバインドを保持する機能を有します。
 * HTMLフォームコンポーネントは本クラスを継承して作成します。
 *
 * @OriginalName DolphinComponent
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
abstract class DolphinComponent {
    
    protected $name;
    protected $label;
    protected $values = array();
    
    /**
     * コンストラクタです。
     *
     * @param $name
     * @return DolphinComponent
     */
    public function __construct($name, $label) {
        
        $this->name = $name;
        $this->label = $label;
        
    }
    
    /**
     *
     * @param $value
     */
    public function setValue($values) {
        
        $this->values = $values;
        
    }
    
    /**
     *
     * @return unknown_type
     */
    public function getValue() {
        
        return $this->values;
        
    }
    
    /**
     *
     * @return unknown_type
     */
    public function getName() {
        
        return $this->name;
        
    }
    
    /**
     * コンポーネントのユーザ対話用表現を返却する抽象メソッドです。
     *
     * @return string
     */
    abstract public function getView();
    
    /**
     * ユーザ入力とプログラム表現の値変換を行う抽象メソッドです。
     *
     * @param array $post
     * @return mixed
     */
    abstract public function doRegular($post);
    
}

/**
 * HTML の1行テキストボックスを表現する DolphinComponent です。
 *
 * @OriginalName DolphinTextBox
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
class DolphinTextBox extends DolphinComponent {
    
    private $maxLength;
    
    /**
     * コンストラクタです。
     *
     * @param unknown_type $name
     * @param unknown_type $label
     * @return unknown_type
     */
    public function __construct($name, $label, $maxLength = 60) {
        
        parent::__construct($name, $label);
        $this->maxLength = $maxLength;
        
    }
    
    /**
     * @see DolphinComponent#getHTML()
     */
    public function getView() {
        
        $view = "<th>{$this->label}</th><td>";
        $suffix = '';
        if(count($this->values) > 1) {
            $suffix = '[]';
        }
        
        foreach($this->values as $value) {
            $view .=
              '<input type="text" '
              . 'name="' . esc_attr($this->name) . $suffix . '" '
              . 'value="' . esc_attr($value) . '" '
              . 'maxlength="' . esc_attr($this->maxLength) . '" '
              . 'size="' . esc_attr($this->maxLength) . '" /><br />';
        }
        $view .= '</td>' . "\n";
        
        return $view;
        
    }
    
    /**
     * @see DolphinComponent#isValidate()
     */
    public function doRegular($posts) {
        
        $result = $posts;
        return $result;
        
    }
    
}

/**
 * HTML のチェックボックスを表現する DolphinComponent です。
 *
 * @OriginalName DolphinCheckBox
 * @version 1.0.0
 * @author hiromasa
 * @modify
 */
class DolphinCheckBox extends DolphinComponent {
    
    /**
     * コンストラクタです。
     *
     * @param $value
     * @param $label
     * @return DolphinCheckBox
     */
    public function __construct($name, $label) {
        
        parent::__construct($name, $label);
                
    }
    
    /**
     * @see DolphinComponent#getView()
     */
    public function getView() {
        
        $view = "<th>{$this->label}</th><td>";
        $suffix = '';
        if(count($this->values) > 1) {
            $suffix = '[]';
        }
        
        foreach($this->values as $value) {
            $checked = '';
            if($value == true) {
                $checked = 'checked';
            }
            $view .=
              '<input type="checkbox" '
              . 'name="' . esc_attr($this->name) . $suffix . '" '
              . $checked . '/><br />';
        }
        $view .= '</td>' . "\n";
        
        return $view;
        
    }
    
   /**
    * @see DolphinComponent#doRegular()
    */
    public function doRegular($posts) {
        
        $result = array();
        foreach($posts as $post) {
           if($post == 'on') {
               array_push($result, true);
           } else {
               array_push($result, false);
           }
        }
        return $result;
        
    }
    
}

/******************************************************************************
 * DolphinPanel - Global Function
 *****************************************************************************/

function add_dolphin(Dolphinable $listener, $index, $role = 'manage_options') {
    
    DolphinController::getInstance()->add($listener, $index, $role);
    
}

function remove_dolphin($index) {
    
    DolphinController::getInstance()->remove($index);
    
}

add_action('init'
    , array(DolphinController::getInstance(), 'notify'), 1);

?>