<?php
/**
 * wp-ezbridge:
 * Convert latitude and longtude to some information from XML
 * (e.g. Area code...)
 * 
 * @author Sa.Ga
 * @version 0.5
 */

/*  Copyright 2006 Sa.Ga

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

$buffer = array();
$bufferPos = 0;
$currentElement = "";
$topLevelNode = "";
$secondLevelNode = "";

/**
 * This file is called only.
 */
if(basename($_SERVER['SCRIPT_FILENAME']) == 'wp-ezbridge.php') {
	die();
}

/**
 * Read XML File to read area information...
 * @param $xmlFile XML File
 * @param $postID Post-ID
 * @param $latitude latitude
 * @param $logtude longtude
 * @param $allowErrors Allow to compute within +/- $allowErrors degree(s) error.
 * @param $topLevelNodeName name of top level node
 * @param $secondLevelNodeName name of second level node
 */
function readXML($xmlFile, $postID, $latitude, $longtude, $allowErrors,
		$topLevelNodeName, $secondLevelNodeName) {
	
	global $wpdb, $topLevelNode, $secondLevelNode;

	$bufferSize = 4096;

	$parser = xml_parser_create();
	if (!$parser) {
		// Error Report
		$errMsg = "XML Parser Error!";
		updateReport($postID, $errMsg);
		return false;
	}

	$topLevelNode = $topLevelNodeName;
	$secondLevelNode = $secondLevelNodeName;

	// Read nearest city area information from XML
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, false);
	xml_set_element_handler($parser,
		'start_element_handler', 'end_element_handler');
	xml_set_character_data_handler($parser, 'character_data_handler');
	xml_set_default_handler($parser, 'default_handler');

	// Open XML Document
	$fp = fopen($xmlFile, "r");
	if ($fp) {
		while ($fin = fread($fp, $bufferSize)) {
			if (!xml_parse($parser, $fin, feof($fp))) {
				// Error Report
				$errMsg = "XML Parser Error!";
				updateReport($postID, $errMsg);
				return false;
			}
		}
	} else {
		// Error Report
		$errMsg = "Failure to open XML file! : " . $xmlFile;
		updateReport($postID, $errMsg);
		return false;
	}
	// Release XML Parser
	xml_parser_free($parser);
	// Compute
	return getNewestAreaInfo($postID, $latitude, $longtude, $allowErrors);
}

/**
 * Get nearest area information from XML buffered data.
 * @param $postID Post-ID
 * @param $latitude latitude
 * @param $logtude longtude
 * @param $allowErrors Allow to compute within +/- $allowErrors degree(s) error.
 */
function getNewestAreaInfo($postID, $latitude, $longtude, $allowErrors) {

	global  $wpdb, $buffer;
	
	// distance * distance
	$latDiff = 0;
	$longDiff = 0;
	$cnt = 0;
	$nearest = -1;	// -1: disabled value...
	$areaID = -1;	// -1: disabled value...

	foreach ($buffer as $cur) {
		$latDiff = $cur["lat"] - $latitude;
		$longDiff = $cur["long"] - $longtude;
		if ((abs($latDiff) <= $allowErrors) && (abs($longDiff) <= $allowErrors)) {
			// the Pythagorean theorem...
			$squareOfDistance = ($latDiff * $latDiff) + ($longDiff * $longDiff);
			if ($nearest < 0 || $nearest > $squareOfDistance) {
				$nearest = $squareOfDistance;
				$areaID = $cnt;
			}
		}
		$cnt++;
	}

	if ($nearest < 0 || $areaID < 0) {
		// Error Report
		$errMsg = "Failure to get nearest area information...";
		updateReport($postID, $errMsg);
		return false;
	}

	// Write area information to metadata.
	$info = '';
	foreach ($buffer[$areaID] as $key => $value) {
		$value = trim($value);
		if ($value != '') {
			$info .= $key . "=". $value . ";";
		}
	}
	updateAreaInfo($postID, $info);
	return true;
}

/**
 * Insert or Update error report to your wordpress database.
 * @param $postID Post-ID
 * @param $errMsg Error Message
 */
function updateReport($postID, $errMsg) {

	global  $wpdb;

	$cnt = $wpdb->get_var("SELECT Count(*) FROM {$wpdb->postmeta} " .
			"WHERE post_id = '{$postID}' AND meta_key = '_wp_eznavi_report' ");
	if ($cnt == null || $cnt == '0') {
		add_post_meta($postID, '_wp_eznavi_report', $errMsg);
	} else {
		$meta_value = $wpdb->get_var("SELECT meta_value FROM {$wpdb->postmeta} " .
				"WHERE post_id = '{$postID}' AND meta_key = '_wp_eznavi_report' LIMIT 1 ");
		update_post_meta($postID, '_wp_eznavi_report', $errMsg, $meta_value);
	}
}

/**
 * Insert or Update nearest area information to your wordpress database.
 * @param $postID Post-ID
 * @param $info nearest area information
 */
function updateAreaInfo($postID, $info) {

	global  $wpdb;

	$cnt = $wpdb->get_var("SELECT Count(*) FROM {$wpdb->postmeta} " .
			"WHERE post_id = '{$postID}' AND meta_key = 'wpotenki_area' ");
	if ($cnt == null || $cnt == '0') {
		add_post_meta($postID, 'wpotenki_area', $info);
	} else {
		$meta_value = $wpdb->get_var("SELECT meta_value FROM {$wpdb->postmeta} " .
				"WHERE post_id = '{$postID}' AND meta_key = 'wpotenki_area' LIMIT 1 ");
		update_post_meta($postID, 'wpotenki_area', $info, $meta_value);
	}
}

/**
 * Start element handler (used by XML Parser)
 * @param $parser XML Parser
 * @param $name Element name
 * @param $attrs Arrays of attributes
 */
function start_element_handler($parser, $name, $attrs) {

	global $buffer, $bufferPos, $topLevelNode, $secondLevelNode, $currentElement;

	if ($name == $topLevelNode) {
		$currentElement = "";
	} else {
		$currentElement = $name;
	}
}

/**
 * End element handler (used by XML Parser)
 * @param $parser XML Parser
 * @param $name Element name
 */
function end_element_handler($parser, $name) {

	global $bufferPos, $currentElement, $secondLevelNode;	

	if ($name == $secondLevelNode) {
		$bufferPos++;
	}
	$currentElement = "";
}

/**
 * Character data handler (used by XML Parser)
 * @param $parser XML Parser
 * @param $chrdata Body String read by XML Parser
 */
function character_data_handler($parser, $chrdata) {

	global $buffer, $bufferPos, $currentElement;

	if ($currentElement != "") {
		$buffer[$bufferPos][$currentElement] = 
			mb_convert_encoding($chrdata, get_settings('blog_charset'), 'auto');
	}
}

/**
 * Default handler (used by XML Parser)
 * @param $parser XML Parser
 * @param $nodedata Node data
 */
function default_handler($parser, $nodedata) {
}

?>
